from pydantic import BaseModel, Field
from typing_extensions import Literal, Optional

from Schemas.enumeration.enums import StudiesLevel, LocalType
from Schemas.models.model import AnnexeModel
from Schemas.type.types import Year


class TeacherFurniture(BaseModel):
    teacher_desk_with_chair: int = Field(..., description="Number of teacher's desks with chairs")
    cupboard_or_closet: int = Field(..., description="Number of cupboards or closets")


Good_or_bad = Literal['Bon / Acceptable', 'Mauvais']


class Office(AnnexeModel):
    name: str
    study_years: StudiesLevel
    local_type: LocalType
    year_of_commissioning: Year
    is_unused: bool
    wall_material: Literal["En dur", "Semi-dur / banco", "Autre: Planche / bambou", "Sans mur"]
    wall_condition: Good_or_bad
    roof_material: Literal["Tôles", "Tuiles / Fibro ciment / Dalles", "Banco", "Paille", "Sans toit"]
    roof_condition: Good_or_bad
    door_material: Literal["Métallique", "Tôle / Bois", "Non installées"]
    wall_nature: Literal["Persiennes", "Volets", "Claustras", "Non installées"]
    funding: Literal["Collectivités locales", "APE", "Aide extérieure", "Autres / Non déterminé"]
    teacher_furniture: TeacherFurniture
    # add student_furniture
    surface_area: float = Field(..., description="Surface area of the office (in m²)")
    blackboard: int = Field(..., description="Number of blackboards")


class OfficeAndFurniture(AnnexeModel):
    """Se réfère à la section Locaux Et Mobiliers"""
    offices: Optional[list[Office]]




from pydantic import Field
from typing_extensions import Annotated, Literal

Year = Annotated[int, Field(strict=True, ge=1000, le=9999)]
Length = Annotated[int, Field(strict=True, ge=0)]
Sexe = Literal["M", "F"]


from typing_extensions import TypedDict, Optional

from Schemas.models.model import AnnexeModel


class SexeCount(TypedDict):
    M: Optional[int]
    F: Optional[int]


class ExamCategoryResults(AnnexeModel):
    inscrits: SexeCount
    ont_compose: SexeCount
    admis: SexeCount


class ExamResult(AnnexeModel):
    """"""
    cep: ExamCategoryResults
    cep_vie_active: ExamCategoryResults



from datetime import date

from pydantic import BaseModel, Field, EmailStr, constr
from typing_extensions import Annotated, Optional

from Schemas.enumeration.enums import SchoolStatus, SchoolKind
from Schemas.models.model import AnnexeModel
from Schemas.type.types import Year


class ReferenceYear(AnnexeModel):
    """Contient les informations de référence de l'année scolaire"""

    creation_year: Optional[Year]
    creation_reference: Optional[str]
    creation_year_reference_date: Optional[date]

    extension_year: Optional[Year]
    extension_reference: Optional[str]
    extension_year_reference_date: Optional[date]


class LocaPeda(BaseModel):
    """Se réfère à la section Localisation dans le Réseau d'Animation Pédagogique"""
    region_pedagogique: constr(pattern=r'^RP\d{0,2}$')
    circonscription_scolaire: Optional[str]
    zone_pedagogique: Optional[Annotated[int, Field(strict=True, ge=1, le=6)]]
    unite_pedagogique: Optional[str]


class LocaTerritoire(BaseModel):
    """Se réfère à la section Localisation dans l'Administration Territorialee"""
    departement: Optional[str]
    commune: Optional[str]
    arrondissement: Optional[str]
    village: Optional[str]
    telephone: Optional[str]
    email: Optional[EmailStr]


class IdAndLocation(BaseModel):
    """Se réfère à la section Identification et localisation de l'école"""
    school_status: Optional[SchoolStatus]
    school_kind: Optional[SchoolKind]
    reference_year: ReferenceYear
    loca_peda: LocaPeda
    loca_territoire: LocaTerritoire



from typing import Dict, Any

from bson import ObjectId
from pydantic import BaseModel, GetJsonSchemaHandler
from pydantic_core import CoreSchema


class PydanticObjectId(ObjectId):
    """
    ObjectId field. Compatible with Pydantic.
    """

    @classmethod
    def __get_validators__(cls):
        yield cls.validate

    @classmethod
    def validate(cls, v, self_instance=None):
        return PydanticObjectId(v)

    @classmethod
    def __modify_schema__(cls, field_schema: dict):
        field_schema.update(
            type="string",
            examples=["5eb7cf5a86d9755df3a6c593", "5eb7cfb05e32e07750a1756a"],
        )

    @classmethod
    def __get_pydantic_json_schema__(
            cls, core_schema: CoreSchema, handler: GetJsonSchemaHandler
    ) -> Dict[str, Any]:
        json_schema = super().__get_pydantic_json_schema__(core_schema, handler)
        json_schema = handler.resolve_ref_schema(json_schema)
        json_schema.update(examples='examples')
        return json_schema


BaseModel.model_config["json_encoders"] = {PydanticObjectId: str}



import inspect
from typing import Optional, Any

from pydantic import BaseModel, Field
from datetime import date, datetime
from Schemas.models.objectid import PydanticObjectId


class Model(BaseModel):
    id: Optional[PydanticObjectId] = Field(None, alias="_id")
    database: Optional[Any] = None

    @staticmethod
    def convert_values(obj):
        if isinstance(obj, (date, datetime)):
            return obj.isoformat()  # Convertir les dates en ISO
        elif isinstance(obj, list):
            return [Model.convert_values(item) for item in obj]  # Appliquer la conversion à chaque élément d'une liste
        elif isinstance(obj, dict):
            return {key: Model.convert_values(value) for key, value in
                    obj.items()}  # Appliquer la conversion à chaque élément d'un dictionnaire
        else:
            return obj  # Retourner l'objet inchangé s'il est sérialisable
            # Appliquer la conversion récursive au dictionnaire principal

    def dict(self, *args, **kwargs):
        kwargs["exclude"] = kwargs.get("exclude", set()) | {"database"}
        properties = [prop_name for prop_name, prop in inspect.getmembers(self.__class__) if isinstance(prop, property)]
        for prop in properties:
            getattr(self, prop)
        return Model.convert_values(super().dict(*args, **kwargs))

    def set_db(self, database):
        self.database = database

    def to_bson(self, to_exclude=None):
        if to_exclude is None:
            to_exclude = set()
        to_exclude.add("database")
        data = self.dict(by_alias=True, exclude=to_exclude)
        if "_id" in data and data["_id"] is None:
            data.pop("_id")
        return data


class AnnexeModel(BaseModel):
    ...



from typing import Optional

from typing_extensions import Literal

from Schemas.models.model import AnnexeModel


class ParticipatorySchoolManagement(AnnexeModel):
    """"""
    is_ape_office_functional: Optional[bool]
    has_all_required_members: Optional[bool]
    num_meetings_previous_year: Optional[int]
    num_general_assemblies_previous_year: Optional[int]
    num_assemblies_budget: Optional[int]
    num_assemblies_academic_results: Optional[int]
    other_assembly_themes: Optional[
        list[Literal["Environnement", "Santé",
        "Kit Scolaire", "Maintien des enfants à l'école"]]]
    result_communication_methods: Optional[
        list[Literal[
            "Carnet de correspondance",
            "Réunion APE",
            "AG",
            "Echange avec les parents concerné"
        ]]]
    has_development_education_projects: Optional[bool]
